# Copyright (c) 2020, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=invalid-name, too-few-public-methods

"""zipfile template"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import logging # pylint: disable=unused-import

import hpccm.base_object

class zipfile(hpccm.base_object):
    """zipfile template"""

    def __init__(self, **kwargs):
        """Initialize zip template"""

        super(zipfile, self).__init__(**kwargs)

    def unzip_step(self, zipfile=None, directory=None):
        """Generate zip command line string"""

        if not zipfile:
            logging.error('zip file is not defined')
            return ''

        if directory:
            return 'mkdir -p {0} && unzip -d {0} {1}'.format(directory,
                                                             zipfile)
        else:
            return 'unzip {}'.format(zipfile)
